#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <linux/limits.h>
#include <BBB.h>

static BBB_pwm *pwm;

void transmit_timecode(int *send_data)
{
	int i;
	
	for( i = 0; i < 60; i++ ) {
		switch( send_data[i] ) {
			
		 case -1: /* Marker */
			pwm->start(pwm);
			usleep(200 * 1000 - 2);
			pwm->stop(pwm);
			usleep(800 * 1000 - 2);
			break;
			
		 case 1: /* 1 */
			pwm->start(pwm);
			usleep(500 * 1000 - 2);
			pwm->stop(pwm);
			usleep(500 * 1000 - 2);
			break;
		
		  case 0: /* 0 */
			pwm->start(pwm);
			usleep(800 * 1000 - 2);
			pwm->stop(pwm);
			usleep(200 * 1000 - 2);
			break;
		}
	}
}

void wait_for_next_minute( void )
{
	struct timeval tv;
	time_t jt;
	struct tm *tmt;
	suseconds_t usec;
	int sec;

        jt = time(NULL);
        tmt = localtime(&jt);
	gettimeofday(&tv, NULL);
        usec = (suseconds_t)1000000 - (tv.tv_usec % 1000000);
	sec = tmt->tm_sec;
	if( sec == 0 ) {
		printf( "Differ = %lu usec\n", tv.tv_usec );
		if( tv.tv_usec < 500 * 1000 ) return;
	}
	// wait for next second
	usleep(usec - 1);
	if( ++sec > 59 ) return;

	// wait for next minute;
	usec = (60 - sec) * 1000 * 1000;
	fprintf(stdout , "Wait for next minute, %u usec\n",  usec );
	usleep(usec - 1);
}

void signal_handler(int sig)
{
	BBB_close_pwm(pwm);
	exit(0);
}

int main(int argc, char *argv[])
{
	int i;
	int f_daemon = 0;
	float freq = 40000.0;
	char pin_name[256] = "P9_22";
    
	/* options */
	if( argc > 1 ) {
		for( i = 1; i < argc; i++ ) {
			char *param = argv[i];
			
			if( strcmp( param, "-d" ) == 0 ) f_daemon = 1;
			else if( strcmp( param, "-p" ) == 0 ) {
				if( ++i >= argc ) {
					fprintf(stderr, "-p is require pin name\n" );
					exit(0);
				}
				strncpy(pin_name, argv[i], 255 );
				fprintf(stderr, "pin name is %s\n", pin_name );
			}
			else if( strcmp( param, "-w" ) == 0 ) {
				freq = 60000.0;
				fprintf(stderr, "Base frequency set to 60kHz\n");
			}
			else if( strcmp( param, "-h" ) == 0 ) {
				fprintf( stderr, 
				 "Usage %s [OPTION]\n"
				 " OPTIONS are follows\n"
				 "  -d           daemonize\n"
				 "  -p pin_name  PWM use pin_name (default P9-22)\n"
				 "  -w           change frequency to 60kHz (default 40kHz)\n"
				 "  -h           display this text and exit\n",
 				 argv[0] );
				exit(0);
			}
		}
	}
	/* set up PWM */
	pwm = BBB_open_pwm( pin_name );
	if( pwm == NULL ) {
		fprintf(stderr, "Can not initialize %s to PWM", pin_name );
		exit(0);
	}
	pwm->stop(pwm);
	pwm->set_frequency( pwm, freq );
	pwm->set_duty(pwm, 0.5);
	
	/* signal handle */
	signal( SIGKILL, signal_handler );
	signal( SIGTERM, signal_handler );
	signal( SIGINT, signal_handler );
	/* daemonize */
	if( f_daemon ) daemon( 0, 0 );

	/* loop start */
	while (1) {
		int t;
		time_t jt;
		struct tm *tmt;
		int send_data[60];
		
		wait_for_next_minute();
		
		jt = time(NULL);
    		tmt = localtime(&jt);
		fprintf(stdout, 
		 "Current time: %04d/%02d/%02d %02d:%02d:%02d\n",
		 tmt->tm_year + 1900,
 		 tmt->tm_mon+1,
		 tmt->tm_mday,
		 tmt->tm_hour,
		 tmt->tm_min, 
		 tmt->tm_sec);

		send_data[0] = -1;
    	
		// minute (10)
 		t = tmt->tm_min / 10;
		send_data[1] = t & 4 ? 1 : 0;
		send_data[2] = t & 2 ? 1 : 0;
		send_data[3] = t & 1 ? 1 : 0;
		send_data[4] = 0;
		
		// minute (1)
		t = tmt->tm_min - t * 10;
		send_data[5] = t & 8 ? 1 : 0;
		send_data[6] = t & 4 ? 1 : 0;
		send_data[7] = t & 2 ? 1 : 0;
		send_data[8] = t & 1 ? 1 : 0;

		// Marker
		send_data[9] = -1;
		send_data[10] = 0;
		send_data[11] = 0;
		
		// Hour(10)
		t = tmt->tm_hour / 10;
		send_data[12] = t & 2 ? 1 : 0;
		send_data[13] = t & 1 ? 1 : 0;
		
		send_data[14] = 0;

		// Hour(1)
		t = tmt->tm_hour - t * 10;
		send_data[15] = t & 8 ? 1 : 0;
		send_data[16] = t & 4 ? 1 : 0;
		send_data[17] = t & 2 ? 1 : 0;
		send_data[18] = t & 1 ? 1 : 0;
    
		// Marker
   		send_data[19] = -1;
   		
   		send_data[20] = 0;
		send_data[21] = 0;
    	
		// year(100)
		t = (tmt->tm_yday + 1) / 100;
		send_data[22] = t & 2 ? 1 : 0;
		send_data[23] = t & 1 ? 1 : 0;
		send_data[24] = 0;
    
		// year(10)
		t = (tmt->tm_yday + 1) / 10 - t * 10;
		send_data[25] = t & 8 ? 1 : 0;
		send_data[26] = t & 4 ? 1 : 0;
		send_data[27] = t & 2 ? 1 : 0;
		send_data[28] = t & 1 ? 1 : 0;

    		// Marker
		send_data[29] = -1;
	
		// year(1)
		t = (tmt->tm_yday + 1) % 100 - t * 10;
		send_data[30] = t & 8 ? 1 : 0;
		send_data[31] = t & 4 ? 1 : 0;
		send_data[32] = t & 2 ? 1 : 0;
 		send_data[33] = t & 1 ? 1 : 0;

		send_data[34] = 0;
		send_data[35] = 0;
		
		// Checksum
		t = 0;
		for(i = 12 ;i < 19 ; i++) {
			t += send_data[i] == 1 ? 1 : 0;
		}
		send_data[36]= t & 1;
    	
		// Checksum
    		t = 0;
   		for (i = 1 ; i < 9 ; i++) {
			t += send_data[i] == 1 ? 1 : 0;
		}
		send_data[37]= t & 1;

		send_data[38] = 0;
    		
		// Marker
    		send_data[39] = -1;
		send_data[40] = 0;
    
		// year(10)
		t = (tmt->tm_year % 100) / 10;
   		send_data[41]= t & 8 ? 1 : 0;
    		send_data[42]= t & 4 ? 1 : 0;
    		send_data[43]= t & 2 ? 1 : 0;
    		send_data[44]= t & 1 ? 1 : 0;
    	
    		// year(1)
		t = (tmt->tm_year % 100 ) - t * 10;
   		send_data[45] = t & 8 ? 1 : 0;
    		send_data[46] = t & 4 ? 1 : 0;
    		send_data[47] = t & 2 ? 1 : 0;
    		send_data[48] = t & 1 ? 1 : 0;
    	
		// Marker
    		send_data[49] = -1;

		// day of week
		t = tmt->tm_wday;
		send_data[50] = t & 4 ? 1 : 0;
   		send_data[51] = t & 2 ? 1 : 0;
    		send_data[52] = t & 1 ? 1 : 0;
    	
    		send_data[53] = 0;
    		send_data[54] = 0;
    	
 		send_data[55] = 0;
		send_data[56] = 0;
		send_data[57] = 0;
		send_data[58] = 0;

    		// Marker(last)
		send_data[59] = -1;
		transmit_timecode(send_data);
	}
}
