#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <dirent.h>
#include <unistd.h>
#include <linux/limits.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "BBB.h"

#define SYS_DEVICES_DIR	"/sys/devices/"
#define BBB_PWM_CAPE	"am33xx_pwm"

static char capemgr_dir[PATH_MAX] = SYS_DEVICES_DIR;
static char ocp_dir[PATH_MAX] = SYS_DEVICES_DIR;
static char slots[PATH_MAX];

static struct dirent *BBB_find_sysdir( char *dest, char *name )
{
	struct dirent **list;
	int n;

	n = scandir( dest, &list, NULL, NULL );
	if( n < 0 ) return NULL;
	while( n-- ) {
		if( strstr(list[n]->d_name, name ) != NULL )
			return list[n];
	}
	return NULL;
}

static int BBB_sysdir_init(void)
{
	struct dirent *ent = BBB_find_sysdir(SYS_DEVICES_DIR, "ocp." );
	if( ent == NULL ) return -1;
	strcat(ocp_dir, ent->d_name );
	strcat(ocp_dir, "/" );
	free(ent);

	ent = BBB_find_sysdir(SYS_DEVICES_DIR, "bone_capemgr." );
	if( ent == NULL ) return -1;
	strcat(capemgr_dir, ent->d_name);
	strcat(capemgr_dir, "/");
	free(ent);
	strcpy( slots, capemgr_dir );
	strcat( slots, "slots" );

	return 0;
}

static int BBB_find_cape( char *name )
{
	FILE *fp;
	int ffind = 0;

	fp = fopen( slots, "rw" );
	if( fp == NULL ) return -1;
	while(! feof(fp) ) {
		char line[1024];
		if( fgets(line, 1024, fp) == NULL ) break;
		if( strstr(line, name) != NULL ) ffind++;
	}
	fclose(fp);
	return ffind;
}

static int BBB_connect_pwm_cape(void)
{
	int ffind = BBB_find_cape(BBB_PWM_CAPE);
	if( ffind < 1 ) {
		FILE *fp = fopen( slots, "w" );
		if( fp == NULL ) return -1;
		fputs(BBB_PWM_CAPE , fp );
		fclose(fp);
	}
	usleep(1000);
	ffind = BBB_find_cape(BBB_PWM_CAPE);
	if( ffind < 1 ) return -1;
	return 0;
}

static int BBB_pwm_set_frequency( BBB_pwm *pwm, float hz )
{
	char cvalue[256];
	unsigned long value = (unsigned long)(1000*1000*1000.0 / hz);

	sprintf(cvalue , "%lu" , value );
	write( pwm->period_fd , cvalue, strlen( cvalue ) );
	lseek( pwm->period_fd , 0 , SEEK_SET );

	pwm->current_period = value;
	return 0;
}

static int BBB_pwm_set_duty( BBB_pwm *pwm, float duty )
{
	char cvalue[256];

	if( (duty > 1.0) || (duty < 0.0) ) return -1;

	unsigned long value = (unsigned long)((float)(pwm->current_period) * duty);

	sprintf(cvalue, "%lu", value );
	write( pwm->duty_fd, cvalue, strlen( cvalue ) );
	lseek( pwm->duty_fd, 0 , SEEK_SET);
	
	pwm->current_duty = value;
	return 0;
}

static int BBB_pwm_start( BBB_pwm *pwm ) 
{
	write( pwm->run_fd , "1", 1 );
	lseek( pwm->run_fd, 0 , SEEK_SET );
	return 0;
}

static int BBB_pwm_stop( BBB_pwm *pwm )
{
	write( pwm->run_fd, "0" , 1 );
	lseek( pwm->run_fd, 0, SEEK_SET );
	return 0;
}

BBB_pwm *BBB_open_pwm( char *name ) 
{
	char pwm_entry_name[256] = "pwm_test_";
	char buffer[1024];

	if( BBB_sysdir_init() < 0 ) return NULL;
	if( BBB_connect_pwm_cape() < 0 ) return NULL;

	strcat(pwm_entry_name, name );
	strcat(pwm_entry_name, "." );
	struct dirent *ent = BBB_find_sysdir( ocp_dir, pwm_entry_name );
	if( ent == NULL ) {
		char pwm_cape_name[256] = "bone_pwm_";
		strcat( pwm_cape_name, name );
		FILE *fp = fopen( slots, "w" );
		if( fp == NULL ) return NULL;
		fputs(pwm_cape_name, fp );
		fclose(fp);
		usleep(1000);
		ent =  BBB_find_sysdir( ocp_dir, pwm_entry_name );
		if( ent == NULL ) return NULL;
	}
	/* success */
	char pwm_dir[1024];
	char pwm_period[1024];
	char pwm_duty[1024];
	char pwm_run[1024];

	strcpy( pwm_dir , ocp_dir );
	strcat( pwm_dir , ent->d_name );
	strcat( pwm_dir , "/" );
	free(ent);

	strcpy( pwm_period, pwm_dir );
	strcat( pwm_period, "period" );

	strcpy( pwm_duty , pwm_dir );
	strcat( pwm_duty , "duty" );

	strcpy( pwm_run , pwm_dir );
	strcat( pwm_run , "run" );

	int period_fd = open( pwm_period, O_RDWR | O_SYNC );
	if( period_fd < 0 ) return NULL;

	int duty_fd = open( pwm_duty , O_RDWR | O_SYNC );
	if( duty_fd < 0 ) {
		close (period_fd);
		return NULL;
	}
	int run_fd = open( pwm_run , O_RDWR | O_SYNC );
	if( run_fd < 0 ) {
		close(period_fd);
		close(duty_fd);
		return NULL;
	}

	if( read( period_fd , buffer, 1024 ) < 1 ) {
		close(period_fd);
		close(duty_fd);
		close(run_fd);
		return NULL;
	}
	unsigned long current_period = atol(buffer);
	lseek(period_fd, 0, SEEK_SET );

	if( read( duty_fd , buffer, 1024 ) < 1 ) {
		close(period_fd);
		close(duty_fd);
		close(run_fd);
		return NULL;
	}
	unsigned long current_duty = atol(buffer);
	lseek(period_fd, 0, SEEK_SET );
	
	BBB_pwm *pwm = (BBB_pwm *)malloc(sizeof(BBB_pwm));
	if( pwm != NULL ) {
		char buffer[1024];

		strcpy( pwm->name , name );
		strcpy( pwm->sysdir, pwm_dir );

		pwm->current_period = current_period;
		pwm->current_duty = current_duty;

		pwm->period_fd = period_fd;
		pwm->duty_fd = duty_fd;
		pwm->run_fd = run_fd;

		pwm->set_frequency = BBB_pwm_set_frequency;
		pwm->set_duty = BBB_pwm_set_duty;
		pwm->start = BBB_pwm_start;
		pwm->stop = BBB_pwm_stop;
	}
	else {
		close(period_fd);
		close(duty_fd);
		close(run_fd);
	}
	return pwm;
}


void BBB_close_pwm( BBB_pwm *pwm )
{
	pwm->stop(pwm);
	close(pwm->period_fd);
	close(pwm->run_fd);
	close(pwm->duty_fd);
	free(pwm);
}

#include <BBB_GPIO.h>

#define GPIO_SYS_DIR	"/sys/class/gpio/"
#define GPIO_SYS_EXPORT	"/sys/class/gpio/export"
#define GPIO_SYS_UNEXPORT	"/sys/class/gpio/unexport"

int BBB_gpio_set_direction( BBB_gpio *gpio, BBB_GPIO_DIRECTION d)
{
	char direction_path[PATH_MAX];
	
	strcpy( direction_path, gpio->path );
	strcat( direction_path, "direction" );
	
	FILE *fp = fopen(direction_path, "w" );
	if( fp == NULL ) return -1;
	
	if( d == IN )
		fputs( "in", fp );
	else if( d == OUT )
		fputs( "out", fp );
	
	fclose(fp);
	return 0;
}

int BBB_gpio_set_edge( BBB_gpio *gpio, BBB_GPIO_TRIGGER t)
{
	char edge_path[PATH_MAX];
	
	strcpy( edge_path, gpio->path );
	strcat( edge_path, "edge" );
	
	FILE *fp = fopen(edge_path, "w" );
	if( fp == NULL ) return -1;
	
	if( t == FALL ) 
		fputs( "falling", fp );
	else if( t == RISE )
		fputs( "rising" , fp );
	else if( t == BOTH )
		fputs( "both", fp );
	
	fclose(fp);
	
	return 0;
}

int BBB_gpio_set_active_low(BBB_gpio *gpio, int b )
{
	char al_path[PATH_MAX];
	
	strcpy( al_path, gpio->path );
	strcat( al_path, "active_low" );
	
	FILE *fp = fopen(al_path, "w" );
	if( fp == NULL ) return -1;
	
	if( b ) 
		fputs( "1" , fp );
	else
		fputs( "0" , fp );
	
	fclose(fp);
	return 0;
}

int BBB_gpio_get(BBB_gpio *gpio)
{
	char buffer[1024];
	
	read( gpio->value_fd, buffer, 1024 );
	lseek( gpio->value_fd, 0, SEEK_SET );
	
	return atoi( buffer );
}

int BBB_gpio_put(BBB_gpio *gpio, int val )
{
	if( val )
		write( gpio->value_fd, "1", 1 );
	else
		write( gpio->value_fd, "0", 1 );

	lseek( gpio->value_fd, 0, SEEK_SET );
	
	return 0;
}

static struct dirent *BBB_find_gpio_dir(char *name)
{
	struct dirent **list;
	int n;

	n = scandir( GPIO_SYS_DIR, &list, NULL, NULL );
	if( n < 0 ) return NULL;
	while( n-- ) {
		if( strcmp(list[n]->d_name, name ) == 0 )
			return list[n];
	}
	return NULL;
}

BBB_gpio *BBB_open_gpio( char *name )
{
	int num;
	struct dirent *dir = NULL;
	
	for( num=0; num < NUM_OF_PINS ; num++ ) {
		if( strcmp( _GPIO_PINS[num].name, name ) == 0 ) break;
	}
	if( num == NUM_OF_PINS ) return NULL;
	if( _GPIO_PINS[num].channel < 0 ) return NULL;
	
	int gpio_no = _GPIO_PINS[num].channel * 32 + _GPIO_PINS[num].bit;
	
	char gpio_dir[8];
	sprintf( gpio_dir, "gpio%d" , gpio_no );
	
	dir = BBB_find_gpio_dir( gpio_dir );
	if( dir == NULL ) {
		FILE *fp = fopen( GPIO_SYS_EXPORT, "w" );
		if( fp == NULL ) return NULL;
		fprintf( fp, "%d", gpio_no );
		fclose(fp);
		usleep(1000);
		dir = BBB_find_gpio_dir( gpio_dir );
		if( dir == NULL ) return NULL;
	}
	free( dir );
	
	char value_path[1024];
	strcpy( value_path, GPIO_SYS_DIR );
	strcat( value_path, gpio_dir );
	strcat( value_path, "/value" );
	
	int value_fd = open( value_path, O_RDWR | O_SYNC );
	if( value_fd < 0 ) return NULL;
	
	/* success */
	
	BBB_gpio *gpio = (BBB_gpio *)malloc( sizeof( BBB_gpio ) );
	if( gpio == NULL ) return;
	
	gpio->pin = &(_GPIO_PINS[num]);
	strcpy( gpio->path, GPIO_SYS_DIR );
	strcat( gpio->path, gpio_dir );
	strcat( gpio->path, "/" );
	
	gpio->value_fd = value_fd;
	
	gpio->set_direction = BBB_gpio_set_direction;
	gpio->set_edge = BBB_gpio_set_edge;
	gpio->set_active_low = BBB_gpio_set_active_low;
	gpio->get = BBB_gpio_get;
	gpio->put = BBB_gpio_put;
	
	return gpio;
}

int BBB_close_gpio( BBB_gpio *gpio )
{
	int gpio_no = gpio->pin->channel * 32 + gpio->pin->bit;
	close( gpio->value_fd );
	FILE *fp = fopen( GPIO_SYS_UNEXPORT , "w" );
	fprintf( fp, "%d", gpio_no );
	fclose(fp);
	free( gpio );
}
