#ifndef _BBB_GPIO_H

#define _BBB_GPIO_H 1

typedef struct _BBB_gpio_pins_t {
	char	name[6];
	int		channel;
	int		bit;
} _BBB_gpio_pins;

typedef enum BBB_GPIO_DIRECTION_t {
	IN,
	OUT,
} BBB_GPIO_DIRECTION;

typedef enum BBB_GPIO_TRIGGER_t {
	RISE,
	FALL,
	BOTH,
} BBB_GPIO_TRIGGER;


typedef struct BBB_gpio_t {
	_BBB_gpio_pins *pin;
	char path[PATH_MAX];
	int value_fd;
	int (*set_direction)(struct BBB_gpio_t *, BBB_GPIO_DIRECTION );
	int (*set_edge)(struct BBB_gpio_t *, BBB_GPIO_TRIGGER );
	int (*set_active_low)(struct BBB_gpio_t *,int);
	int (*get)(struct BBB_gpio_t *);
	int (*put)(struct BBB_gpio_t *,int);
} BBB_gpio;

/* public methods */
BBB_gpio *BBB_open_gpio( char *name );
int BBB_close_gpio( BBB_gpio *);
/** **/

#define NUM_OF_PINS	92

static _BBB_gpio_pins _GPIO_PINS[NUM_OF_PINS] = {
	{ "P8_1", -1,-1 },
	{ "P8_2", -1,-1 },
	{ "P8_3", 1, 6 },
	{ "P8_4", 1, 7 },
	{ "P8_5", 1, 2 },
	{ "P8_6", 1, 3 },
	{ "P8_7", 2, 2 },
	{ "P8_8", 2, 3 },
	{ "P8_9", 2, 5 },
	{ "P8_10", 2,  4 },
	{ "P8_11", 1, 13 },
	{ "P8_12", 1, 12 },
	{ "P8_13", 0, 23 },
	{ "P8_14", 0, 26 },
	{ "P8_15", 1, 15 },
	{ "P8_16", 1, 14 },
	{ "P8_17", 0, 27 },
	{ "P8_18", 2, 1  },
	{ "P8_19", 0, 22 },
	{ "P8_20", 1, 31 },
	{ "P8_21", 1, 30 },
	{ "P8_22", 1, 5  },
	{ "P8_23", 1, 4 },
	{ "P8_24", 1, 1 },
	{ "P8_25", 1, 0 },
	{ "P8_26", 1,29 },
	{ "P8_27", 2,22 },
	{ "P8_28", 2,24 },
	{ "P8_29", 2,23 },
	{ "P8_30", 2,25 },
	{ "P8_31", 0,10 },
	{ "P8_32", 0,11 },
	{ "P8_33", 0, 9 },
	{ "P8_34", 2,17 },
	{ "P8_35", 0, 8 },
	{ "P8_36", 2,16 },
	{ "P8_37", 2,14 },
	{ "P8_38", 2,15 },
	{ "P8_39", 2,12 },
	{ "P8_40", 2,13 },
	{ "P8_41", 2,10 },
	{ "P8_42", 2,11 },
	{ "P8_43", 2, 8 },
	{ "P8_44", 2,9 },
	{ "P8_45", 2,6 },
	{ "P8_46", 2,7 },
	
	{ "P9_1", -1,-1 },
	{ "P9_2", -1,-1 },
	{ "P9_3", -1,-1 },
	{ "P9_4", -1,-1 },
	{ "P9_5", -1,-1 },
	{ "P9_6", -1,-1 },
	{ "P9_7", -1 ,-1 },
	{ "P9_8", -1, -1 },
	{ "P9_9", -1, -1 },
	{ "P9_10", -1, -1 },
	{ "P9_11", 0, 30 },
	{ "P9_12", 1, 28 },
	{ "P9_13", 0, 31 },
	{ "P9_14", 1, 18 },
	{ "P9_15", 1, 16 },
	{ "P9_16", 1, 19 },
	{ "P9_17", 0,  5 },
	{ "P9_18", 0,  4 },
	{ "P9_19", 0, 13 },
	{ "P9_20", 0, 12 },
	{ "P9_21", 0, 3 },
	{ "P9_22", 0, 2 },
	{ "P9_23", 1, 17 },
	{ "P9_24", 0, 15 },
	{ "P9_25", 3, 21 },
	{ "P9_26", 0, 14 },
	{ "P9_27", 3, 19 },
	{ "P9_28", 3, 17 },
	{ "P9_29", 3, 15 },
	{ "P9_30", 3, 16 },
	{ "P9_31", 3, 14 },
	{ "P9_32", -1,-1 },
	{ "P9_33", -1,-1 },
	{ "P9_34", -1,-1 },
	{ "P9_35", -1,-1 },
	{ "P9_36", -1,-1 },
	{ "P9_37", -1,-1 },
	{ "P9_38", -1,-1 },
	{ "P9_39", -1,-1 },
	{ "P9_40", -1,-1 },
	{ "P9_41",  0,20 },
	{ "P9_42",  0, 7 },
	{ "P9_43", -1, -1 },
	{ "P9_44", -1, -1 },
	{ "P9_45", -1, -1 },
	{ "P9_46", -1, -1 },
};


#endif

