
#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <BBB.h>
#include <BBB_GPIO.h>
#include <stdint.h>

static const uint32_t GPIO_REGS[] = {
	0x44E07000,     // GPIO0
        0x4804C000,     // GPIO1
        0x481AC000,     // GPIO2
        0x481AE000      // GPIO3
};

#define GPIO_DATAOUT	0x13C
#define GPIO_DATAIN	0x138
#define GPIO_CLEARDATAOUT	0x190
#define GPIO_SETDATAOUT	0x194

int main(void)
{
	char pin_name[] = "P9_11";

	BBB_gpio *gpio = BBB_open_gpio(pin_name);

	if( gpio == NULL ) {
		printf( "error\n" );
		return;
	}
	gpio->set_direction(gpio, OUT);

	int gpio_fd = open("/dev/mem", O_RDWR | O_SYNC );
	if( gpio_fd == 0 ) {
		printf("cant open /dev/mem\n" );
		return;
	}
	volatile uint32_t *gpio_reg = (uint32_t *)mmap( NULL, 0xFFF, 
PROT_READ|PROT_WRITE, MAP_SHARED, gpio_fd, GPIO_REGS[gpio->pin->channel] );
	if( gpio_reg <= 0 ){
		printf( "GPIO Register mapping failed\n" );
		return;
	}

	int i;
	for( i=0; i<20; i++ ) {
		uint32_t value;

		value = gpio_reg[GPIO_DATAOUT/sizeof(uint32_t)];
		value = value ^ ( 1 << gpio->pin->bit );
		gpio_reg[GPIO_DATAOUT/sizeof(uint32_t)] = value;
		value = 1 <<  gpio->pin->bit;

		if( i & 1 )
			gpio_reg[GPIO_SETDATAOUT/sizeof(uint32_t)] = value;
		else
			gpio_reg[GPIO_CLEARDATAOUT/sizeof(uint32_t)] = value;

		sleep(1);
	}
	munmap((void *) gpio_reg, 0xFFF );
	close(gpio_fd);
	BBB_close_gpio(gpio);
}
