#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aki_lcd.h"

int raster_to_lcd( uint8_t *buffer )
{
	int cs, page , x;
	
	for( page = 0; page < 8 ; page++ ) {
		for( cs = 0 ; cs < 2 ; cs++ ) {
			lcd_write_cmd( 0x40 , cs+1 ); /* set address to 0 */
			for( x = 0; x < 8 ; x++ ) {
				unsigned char mask;
				for( mask = 0x80; mask != 0 ; mask = mask >> 1 ) {
					unsigned char data = 0;
					data |= (buffer[cs*8 + x     + page*128] & mask) ? 0x01 : 0;
					data |= (buffer[cs*8 + x+16  + page*128] & mask) ? 0x02 : 0;
					data |= (buffer[cs*8 + x+32  + page*128] & mask) ? 0x04 : 0;
					data |= (buffer[cs*8 + x+48  + page*128] & mask) ? 0x08 : 0;
					data |= (buffer[cs*8 + x+64  + page*128] & mask) ? 0x10 : 0;
					data |= (buffer[cs*8 + x+80  + page*128] & mask) ? 0x20 : 0;
					data |= (buffer[cs*8 + x+96  + page*128] & mask) ? 0x40 : 0;
					data |= (buffer[cs*8 + x+112 + page*128] & mask) ? 0x80 : 0;
					
					lcd_write_display_data( data, page, cs+1 );
				}
			}
		}
	}
	return 0;
}

int main( int argc, char *argv[] )
{
	char buffer[1024];
	
	if( argc < 2 ) {
		fprintf( stderr, "usage: %s FILE.pbm\n" , argv[0] );
		return -1;
	}
	
	if( lcd_init() < 0 ) {
		fprintf( stderr, "lcd_init failed\n" );
		return -1;
	}
	
	FILE *fp = fopen( argv[1], "rw" );
	if( fp == NULL ) {
		fprintf( stderr, "%s open failed.\n" , argv[1] );
		return -1;
	}
	
	/* check P4 header */
	fgets( buffer, 1024, fp );
	if( strcmp( buffer, "P4\n") == 0 ) {
		fgets( buffer, 1024, fp );
		if( strcmp( buffer, "128 64\n") == 0 ) {
			fread( buffer, 1024, 1, fp );
			raster_to_lcd( buffer);
		}
		else fprintf( stderr, "Size = %s\n", buffer );
	}
	else fprintf( stderr, "Header = %s\n" , buffer );
	fclose(fp);
	
	return 0;
}
